package com.typowebguide.directory;

import java.io.Serializable;
import java.util.Map;
import java.util.NavigableMap;
import java.util.NavigableSet;
import java.util.TreeMap;
import java.util.TreeSet;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.OneToMany;
import javax.persistence.Transient;

@Entity
public class Folder extends Entry implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private static final String SYMBOL_KEY = "#$!";
	private static final String DIGIT_KEY = "0-9";
	private static final int LIST_THRESHOLD = 50;
	
	@OneToMany(cascade=CascadeType.ALL) private Map<String,Entry> children = new TreeMap<String,Entry>();
	@Transient private NavigableMap<String,Entry> nonListChildren = new TreeMap<String,Entry>();
	@Transient private NavigableMap<String,NavigableSet<Entry>> listEntries = new TreeMap<String,NavigableSet<Entry>>();
	@Transient private int activeCount = 0;
	@Transient private int listCount = 0;
	
	public Folder() {
		super();
	}
	
	public Folder(String name) {
		this();
		setName(name);
	}
	
	/**
	 * @return the children
	 */
	public NavigableMap<String, Entry> getChildren() {
		return new TreeMap<String, Entry>(this.children);
	}

	/**
	 * @param parent the parent to set
	 */
	public void setParent(Folder parent) {
		super.setParent(parent);
		parent.children.put(getName(), this);
	}
	
	public void changeParent(Folder parent) {
		this.getParent().children.remove(getName());
		this.setParent(parent);
	}

	public void addChild(Entry child) {
		addChild(child, child.getName());
	}
	
	public void addChild(Entry child, String name) {
		child.setParent(this);
		this.children.put(name, child);
		if (child.isListable()) {
			putEntryInLists(child, name);
		}
		else {
			nonListChildren.put(name, child);
		}
	}
	
	private void putEntryInLists(Entry entry, String name) {
		String key = getListKeyForEntry(name);
		if (!listEntries.containsKey(key)) {
			listEntries.put(key, new TreeSet<Entry>());
		}
		listEntries.get(key).add(entry);
	}

	public void removeChild(Entry child, String name) {
		child.setParent(null);
		children.remove(name);
		nonListChildren.remove(name);
		removeEntryInLists(child, name);
	}

	public void removeChild(Entry child) {
		removeChild(child, child.getName());
	}

	public NavigableSet<String> getListKeys() {
		// Defensive copy
		return new TreeSet<String>(listEntries.navigableKeySet());
	}
	
	public boolean isFolderListingActive() {
		return (listCount > LIST_THRESHOLD);
	}
	
//	private void moveEntriesToLists() {
//		NavigableSet<Entry> newEntries = new TreeSet<Entry>();
//		listEntries = new TreeMap<String, NavigableSet<Entry>>();
//		for (Entry entry: children.values()) {
//			if (entry.isListable()) {
//				putEntryInLists(entry);
//			}
//			else {
//				newEntries.add(entry);
//			}
//		}
//	}
	
	private void removeEntryInLists(Entry entry, String name) {
		String key = getListKeyForEntry(name);
		if (listEntries.containsKey(key)) {
			listEntries.get(key).remove(entry);
			if (listEntries.get(key).isEmpty()) {
				listEntries.remove(key);
			}
		}
	}
	
	private String getListKeyForEntry(String name) {
		String key = SYMBOL_KEY;
		char firstChar = name.charAt(0);
		if (Character.isLetter(firstChar)) {
			key = String.valueOf(firstChar);
		}
		else if (Character.isDigit(firstChar)) {
			key = DIGIT_KEY;
		}
		return key;
	}
	
	public int getActiveCount() {
		return activeCount;
	}

	public boolean showViaLists() {
		return activeCount > LIST_THRESHOLD;
	}
	
	public NavigableMap<String, Folder> getSubDirs() {
		NavigableMap<String, Folder> subDirs = new TreeMap<String, Folder>();
		if (children == null) {
			children = new TreeMap<String, Entry>();
		}
		else {
			for (Map.Entry<String, Entry> entry: children.entrySet()) {
				if (entry.getValue() instanceof Folder){
					subDirs.put(entry.getKey(), (Folder) entry.getValue());
				}
			}
		}
		return subDirs;
	}

	
}
